package cn.iocoder.yudao.module.bpm.controller.admin.oa;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.evection.BpmOAEvectionCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.evection.BpmOAEvectionRespVO;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOAEvectionConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAEvectionDO;
import cn.iocoder.yudao.module.bpm.service.oa.BpmOAEvectionService;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertList;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

/**
 * OA 出差申请 Controller
 *
 * @author 符溶馨

 */
@Tag(name = "管理后台 - OA 出差申请")
@RestController
@RequestMapping("/bpm/oa/evection")
@Validated
public class BpmOAEvectionController {

    @Resource
    private BpmOAEvectionService evectionService;

    @Resource
    private AdminUserApi userApi;

    @PostMapping("/create")
    @Operation(summary = "创建请求申请")
    public CommonResult<Long> createEvection(@Valid @RequestBody BpmOAEvectionCreateReqVO createReqVO) {

        return success(evectionService.createEvection(getLoginUserId(), createReqVO));
    }

    @GetMapping("/get")
    @Operation(summary = "获得出差申请")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    public CommonResult<BpmOAEvectionRespVO> getEvection(@RequestParam("id") Long id) {

        BpmOAEvectionDO evection = evectionService.getEvection(id);

        BpmOAEvectionRespVO respVO = BpmOAEvectionConvert.INSTANCE.convert(evection);

        if (CollectionUtil.isNotEmpty(evection.getTogetherUserIds())) {
            List<AdminUserRespDTO> userRespDTOs = userApi.getUserList(respVO.getTogetherUserIds()).getCheckedData();
            respVO.setTogetherUserName(String.join("、", convertList(userRespDTOs, AdminUserRespDTO::getNickname)));
        }

        return success(respVO);
    }

    @GetMapping("/get-list")
    @Operation(summary = "获得我的出差申请")
    public CommonResult<List<BpmOAEvectionRespVO>> getEvectionList() {

        List<BpmOAEvectionDO> evections = evectionService.getEvectionList();

        return success(BeanUtils.toBean(evections, BpmOAEvectionRespVO.class));
    }

    @GetMapping("/getByProcessInstanceId")
    @Operation(summary = "获得出差申请")
    @Parameter(name = "processInstanceId", description = "流程实例编号", required = true, example = "1024")
    public CommonResult<BpmOAEvectionRespVO> getByProcessInstanceId(@RequestParam("processInstanceId") String processInstanceId) {

        BpmOAEvectionDO evection = evectionService.getByProcessInstanceId(processInstanceId);

        BpmOAEvectionRespVO respVO = BpmOAEvectionConvert.INSTANCE.convert(evection);

        if (CollectionUtil.isNotEmpty(evection.getTogetherUserIds())) {
            List<AdminUserRespDTO> userRespDTOs = userApi.getUserList(evection.getTogetherUserIds()).getCheckedData();
            respVO.setTogetherUserName(String.join("、", convertList(userRespDTOs, AdminUserRespDTO::getNickname)));
        }

        return success(respVO);
    }
}
