package cn.iocoder.yudao.module.bpm.dal.dataobject.oa;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import lombok.*;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;

/**
 * OA 合同审批 DO
 *
 * @author 符溶馨
 */
@TableName(value ="bpm_oa_contract", autoResultMap = true)
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@NoArgsConstructor
@AllArgsConstructor
public class BpmOAContractDO extends BaseDO {

    /**
     * 合同审批表单主键
     */
    @TableId
    private Long id;

    /**
     * 申请人的用户编号
     *
     * 关联 AdminUserDO 的 id 属性
     */
    private Long userId;

    /**
     * 合同类型 | 字典值 bpm_oa_contract_type
     */
    private Integer contractType;

    /**
     * 合同名称
     */
    private String contractName;

    /**
     * 合同编号
     */
    private String contractNo;

    /**
     * 签约日期
     */
    private LocalDate signingDate;

    /**
     * 客户编号
     */
    private Long customerId;

    /**
     * 客户名称
     */
    private String customerName;

    /**
     * 客户签约人（联系人ID）
     */
    private Long contactsId;

    /**
     * 客户签约人(联系人)名称
     */
    private String contactsName;

    /**
     * 签约公司编号
     */
    private Long companyId;

    /**
     * 签约人用户编号
     */
    private Long signatoryId;

    /**
     * 合同金额
     */
    private BigDecimal contractMoney;

    /**
     * 已收/已付金额
     */
    private BigDecimal returnMoney;

    /**
     * 已开票金额
     */
    private BigDecimal invoiceMoney;

    /**
     * 产品总金额
     */
    private BigDecimal totalPrice;

    /**
     * 整单优惠金额
     */
    private BigDecimal discountRate;

    /**
     * 商机ID
     */
    private Long businessId;

    /**
     * 合同有效期开始时间
     */
    private LocalDate startDate;

    /**
     * 合同有效期结束时间
     */
    private LocalDate endDate;

    /**
     * 合同状态 | 1已签约 2过期
     */
    private Integer status;

    /**
     * 是否结算 0否 1是
     */
    private Integer settlementFlag;

    /**
     * 审批结果
     * 枚举 {@link BpmProcessInstanceResultEnum}
     * 考虑到简单，所以直接复用了 BpmProcessInstanceResultEnum 枚举，也可以自己定义一个枚举哈
     */
    private Integer result;

    /**
     * 对应的流程编号
     *
     * 关联 ProcessInstance 的 id 属性
     */
    private String processInstanceId;

    /**
     * 附件基本信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<UploadUserFile> fileItems ;
}
