package cn.iocoder.yudao.module.bpm.dal.dataobject.oa;

import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.type.JsonLongSetTypeHandler;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.salary.SalarySubjectItemsVO;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import lombok.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.Set;

/**
 * OA 薪资付款 DO
 *
 * @author 符溶馨
 */
@TableName(value ="bpm_oa_salary", autoResultMap = true)
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class BpmOASalaryDO extends BaseDO {

    /**
     * 薪资付款表单主键
     */
    @TableId
    private Long id;

    /**
     * 申请人的用户编号
     *
     * 关联 AdminUserDO 的 id 属性
     */
    private Long userId;

    /**
     * 申请事由
     */
    private String reason;

    /**
     * 薪资月份
     */
    private String salaryDate;

    /**
     * 公司类型 | 1公司 2工厂
     */
    private Integer deptType;

    /**
     * 付款公司
     */
    private Long companyDeptId;

    /**
     * 付款工厂的部门编号集合
     */
    @TableField(typeHandler = JsonLongSetTypeHandler.class)
    private Set<Long> factoryDeptId;

    /**
     * 员工薪资id集合
     */
    @TableField(typeHandler = JsonLongSetTypeHandler.class)
    private Set<Long> staffSalaryIds;

    /**
     * 付款总额
     */
    private BigDecimal paymentTotal;

    /**
     * 支付人数
     */
    private Integer paymentNumber;

    /**
     * 支付对象
     */
    private String paymentRecipient;


    /**
     * 薪资主体明细
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<SalarySubjectItemsVO> salarySubjectItems;

    /**
     * 薪资付款申请的结果
     *
     * 枚举 {@link BpmProcessInstanceResultEnum}
     * 考虑到简单，所以直接复用了 BpmProcessInstanceResultEnum 枚举，也可以自己定义一个枚举哈
     */
    private Integer result;

    /**
     * 对应的流程编号
     *
     * 关联 ProcessInstance 的 id 属性
     */
    private String processInstanceId;

    /**
     * 附件基本信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<UploadUserFile> fileItems ;
}
