package cn.iocoder.yudao.module.system.service.position;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.position.vo.PositionPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.position.vo.PositionSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.position.PositionDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 岗位信息 Service 接口
 *
 * @author 符溶馨
 */
public interface PositionService {

    /**
     * 创建岗位信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createPosition(@Valid PositionSaveReqVO createReqVO);

    /**
     * 更新岗位信息
     *
     * @param updateReqVO 更新信息
     */
    void updatePosition(@Valid PositionSaveReqVO updateReqVO);

    /**
     * 删除岗位信息
     *
     * @param id 编号
     */
    void deletePosition(Long id);

    /**
     * 获得岗位信息
     *
     * @param id 编号
     * @return 岗位信息
     */
    PositionDO getPosition(Long id);

    /**
     * 获得岗位信息分页
     *
     * @param pageReqVO 分页查询
     * @return 岗位信息分页
     */
    PageResult<PositionDO> getPositionPage(PositionPageReqVO pageReqVO);

    /**
     * 获得开启得 岗位列表
     * @return 岗位列表
     */
    List<PositionDO> getPositionByDeptId(Long deptId);

    /**
     * 获得指定得岗位列表
     *
     * @param ids 岗位编号数组
     * @return 岗位列表
     */
    List<PositionDO> getPositionList(List<Long> ids);
}